@@ -3,7 +3,7 @@
 /*
   The MIT License (MIT)
 
-  Copyright (c) 2007-2017 Einar Lielmanis, Liam Newman, and contributors.
+  Copyright (c) 2007-2018 Einar Lielmanis, Liam Newman, and contributors.
 
   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation files
@@ -31,6 +31,7 @@
   Written by Daniel Stockman (daniel.stockman@gmail.com)
 
 */
+/*jshint strict:false */
 
 var debug = process.env.DEBUG_JSBEAUTIFY || process.env.JSBEAUTIFY_DEBUG ? function() {
     console.error.apply(console, arguments);
@@ -40,10 +41,13 @@ var fs = require('fs'),
     cc = require('config-chain'),
     beautify = require('../index'),
     mkdirp = require('mkdirp'),
-    nopt = require('nopt');
-nopt.invalidHandler = function(key, val, types) {
+    nopt = require('nopt'),
+    glob = require('glob');
+
+nopt.invalidHandler = function(key, val) {
     throw new Error(key + " was invalid with value \"" + val + "\"");
-}
+};
+
 nopt.typeDefs.brace_style = {
     type: "brace_style",
     validate: function(data, key, val) {
@@ -75,18 +79,21 @@ var path = require('path'),
         "space_in_empty_paren": Boolean,
         "jslint_happy": Boolean,
         "space_after_anon_function": Boolean,
+        "space_after_named_function": Boolean,
         "brace_style": "brace_style", //See above for validation
         "unindent_chained_methods": Boolean,
         "break_chained_methods": Boolean,
         "keep_array_indentation": Boolean,
         "unescape_strings": Boolean,
         "wrap_line_length": Number,
-        "wrap_attributes": ["auto", "force", "force-aligned"],
+        "wrap_attributes": ["auto", "force", "force-aligned", "force-expand-multiline", "aligned-multiple", "preserve", "preserve-aligned"],
         "wrap_attributes_indent_size": Number,
         "e4x": Boolean,
         "end_with_newline": Boolean,
         "comma_first": Boolean,
         "operator_position": ["before-newline", "after-newline", "preserve-newline"],
+        "indent_empty_lines": Boolean,
+        "templating": [String, Array],
         // CSS-only
         "selector_separator_newline": Boolean,
         "newline_between_rules": Boolean,
@@ -95,12 +102,14 @@ var path = require('path'),
         "space_around_selector_separator": Boolean,
         // HTML-only
         "max_char": Number, // obsolete since 1.3.5
+        "inline": [String, Array],
         "unformatted": [String, Array],
         "content_unformatted": [String, Array],
         "indent_inner_html": [Boolean],
         "indent_handlebars": [Boolean],
         "indent_scripts": ["keep", "separate", "normal"],
         "extra_liners": [String, Array],
+        "unformatted_content_delimiter": String,
         // CLI
         "version": Boolean,
         "help": Boolean,
@@ -144,6 +153,7 @@ var path = require('path'),
         "A": ["--wrap_attributes"],
         "i": ["--wrap_attributes_indent_size"],
         "W": ["--max_char"], // obsolete since 1.3.5
+        "d": ["--inline"],
         "U": ["--unformatted"],
         "T": ["--content_unformatted"],
         "I": ["--indent_inner_html"],
@@ -163,11 +173,14 @@ var path = require('path'),
         "v": ["--version"],
         "h": ["--help"],
         "f": ["--files"],
+        "file": ["--files"],
         "o": ["--outfile"],
         "r": ["--replace"],
         "q": ["--quiet"]
         // no shorthand for "config"
         // no shorthand for "editorconfig"
+        // no shorthand for "indent_empty_lines"
+        // not shorthad for "templating"
     });
 
 function verifyExists(fullPath) {
@@ -212,7 +225,7 @@ function set_file_editorconfig_opts(file, config) {
             if (eConfigs.max_line_length === "off") {
                 config.wrap_line_length = 0;
             } else {
-                config.wrap_line_length = parseInt(eConfigs.max_line_length);
+                config.wrap_line_length = parseInt(eConfigs.max_line_length, 10);
             }
         }
 
@@ -337,6 +350,8 @@ function usage(err) {
         '  -e, --eol                         Character(s) to use as line terminators.',
         '                                    [first newline in file, otherwise "\\n]',
         '  -n, --end-with-newline            End output with newline',
+        '  --indent-empty-lines              Keep indentation on empty lines',
+        '  --templating                      List of templating languages (auto,none,django,erb,handlebars,php,smarty) ["auto"] auto = none in JavaScript, all in html',
         '  --editorconfig                    Use EditorConfig to set up the options'
     ];
 
@@ -349,12 +364,13 @@ function usage(err) {
             msg.push('  -E, --space-in-empty-paren        Add a single space inside empty paren, ie. f( )');
             msg.push('  -j, --jslint-happy                Enable jslint-stricter mode');
             msg.push('  -a, --space-after-anon-function   Add a space before an anonymous function\'s parens, ie. function ()');
+            msg.push('  --space_after_named_function      Add a space before a named function\'s parens, ie. function example ()');
             msg.push('  -b, --brace-style                 [collapse|expand|end-expand|none][,preserve-inline] [collapse,preserve-inline]');
             msg.push('  -u, --unindent-chained-methods    Don\'t indent chained method calls');
             msg.push('  -B, --break-chained-methods       Break chained method calls across subsequent lines');
             msg.push('  -k, --keep-array-indentation      Preserve array indentation');
             msg.push('  -x, --unescape-strings            Decode printable characters encoded in xNN notation');
-            msg.push('  -w, --wrap-line-length            Wrap lines at next opportunity after N characters [0]');
+            msg.push('  -w, --wrap-line-length            Wrap lines that exceed N characters [0]');
             msg.push('  -X, --e4x                         Pass E4X xml literals through untouched');
             msg.push('  --good-stuff                      Warm the cockles of Crockford\'s heart');
             msg.push('  -C, --comma-first                 Put commas at the beginning of new line instead of end');
@@ -365,16 +381,18 @@ function usage(err) {
             msg.push('  -I, --indent-inner-html           Indent body and head sections. Default is false.');
             msg.push('  -H, --indent-handlebars           Indent handlebars. Default is false.');
             msg.push('  -S, --indent-scripts              [keep|separate|normal] ["normal"]');
-            msg.push('  -w, --wrap-line-length            Wrap lines at next opportunity after N characters [0]');
-            msg.push('  -A, --wrap-attributes             Wrap html tag attributes to new lines [auto|force] ["auto"]');
+            msg.push('  -w, --wrap-line-length            Wrap lines that exceed N characters [0]');
+            msg.push('  -A, --wrap-attributes             Wrap html tag attributes to new lines [auto|force|force-aligned|force-expand-multiline|aligned-multiple|preserve|preserve-aligned] ["auto"]');
             msg.push('  -i, --wrap-attributes-indent-size Indent wrapped tags to after N characters [indent-level]');
             msg.push('  -p, --preserve-newlines           Preserve line-breaks (--no-preserve-newlines disables)');
             msg.push('  -m, --max-preserve-newlines       Number of line-breaks to be preserved in one chunk [10]');
             msg.push('  -U, --unformatted                 List of tags (defaults to inline) that should not be reformatted');
             msg.push('  -T, --content_unformatted         List of tags (defaults to pre) whose content should not be reformatted');
             msg.push('  -E, --extra_liners                List of tags (defaults to [head,body,/html] that should have an extra newline');
+            msg.push('  --unformatted_content_delimiter    Keep text content together between this string [""]');
             break;
         case "css":
+            msg.push('  -b, --brace-style                       [collapse|expand] ["collapse"]');
             msg.push('  -L, --selector-separator-newline        Add a newline between multiple selectors.');
             msg.push('  -N, --newline-between-rules             Add a newline between CSS rules.');
     }
@@ -391,7 +409,7 @@ function usage(err) {
 // main iterator, {cfg} passed as thisArg of forEach call
 
 function processInputSync(filepath) {
-    var data = '',
+    var data = null,
         config = this.cfg,
         outfile = config.outfile,
         input;
@@ -422,17 +440,27 @@ function processInputSync(filepath) {
 
     if (filepath === '-') {
         input = process.stdin;
-        input.resume();
 
         input.setEncoding('utf8');
 
+        input.on('error', function() {
+            throw 'Must pipe input or define at least one file.';
+        });
+
         input.on('data', function(chunk) {
+            data = data || '';
             data += chunk;
         });
 
         input.on('end', function() {
+            if (data === null) {
+                throw 'Must pipe input or define at least one file.';
+            }
             makePretty(fileType, data, config, outfile, writePretty); // Where things get beautified
         });
+
+        input.resume();
+
     } else {
         data = fs.readFileSync(filepath, 'utf8');
         makePretty(fileType, data, config, outfile, writePretty);
@@ -561,6 +589,8 @@ function checkType(parsed) {
 function checkFiles(parsed) {
     var argv = parsed.argv;
     var isTTY = true;
+    var file_params = parsed.files || [];
+    var hadGlob = false;
 
     try {
         isTTY = process.stdin.isTTY;
@@ -570,54 +600,82 @@ function checkFiles(parsed) {
 
     debug('isTTY: ' + isTTY);
 
-    if (!parsed.files) {
-        parsed.files = [];
-    } else {
-        if (argv.cooked.indexOf('-') > -1) {
-            // strip stdin path eagerly added by nopt in '-f -' case
-            parsed.files.some(removeDashedPath);
-        }
-    }
+    // assume any remaining args are files
+    file_params = file_params.concat(argv.remain);
 
-    if (argv.remain.length) {
-        // assume any remaining args are files
-        argv.remain.forEach(function(f) {
-            if (f !== '-') {
-                parsed.files.push(path.resolve(f));
+    parsed.files = [];
+    // assume any remaining args are files
+    file_params.forEach(function(f) {
+        // strip stdin path eagerly added by nopt in '-f -' case
+        if (f === '-') {
+            return;
+        }
+
+        var foundFiles = [];
+        var isGlob = glob.hasMagic(f);
+
+        // Input was a glob
+        if (isGlob) {
+            hadGlob = true;
+            foundFiles = glob(f, {
+                sync: true,
+                absolute: true,
+                ignore: ['**/node_modules/**', '**/.git/**']
+            });
+        } else {
+            // Input was not a glob, add it to an array so we are able to handle it in the same loop below
+            try {
+                testFilePath(f);
+            } catch (err) {
+                // if file is not found, and the resolved path indicates stdin marker
+                if (path.parse(f).base === '-') {
+                    f = '-';
+                } else {
+                    throw err;
+                }
             }
-        });
-    }
+            foundFiles = [f];
+        }
+
+        if (foundFiles && foundFiles.length) {
+            // Add files to the parsed.files if it didn't exist in the array yet
+            foundFiles.forEach(function(file) {
+                var filePath = path.resolve(file);
+                if (file === '-') { // case of stdin
+                    parsed.files.push(file);
+                } else if (parsed.files.indexOf(filePath) === -1) {
+                    parsed.files.push(filePath);
+                }
+            });
+        }
+    });
 
     if ('string' === typeof parsed.outfile && isTTY && !parsed.files.length) {
+        testFilePath(parsed.outfile);
         // use outfile as input when no other files passed in args
         parsed.files.push(parsed.outfile);
         // operation is now an implicit overwrite
         parsed.replace = true;
     }
 
-    if (!parsed.files.length) {
+    if (hadGlob || parsed.files.length > 1) {
+        parsed.replace = true;
+    }
+
+    if (!parsed.files.length && !hadGlob) {
         // read stdin by default
         parsed.files.push('-');
     }
+
     debug('files.length ' + parsed.files.length);
 
-    if (parsed.files.indexOf('-') > -1 && isTTY) {
+    if (parsed.files.indexOf('-') > -1 && isTTY && !hadGlob) {
         throw 'Must pipe input or define at least one file.';
     }
 
-    parsed.files.forEach(testFilePath);
-
     return parsed;
 }
 
-function removeDashedPath(filepath, i, arr) {
-    var found = filepath.lastIndexOf('-') === (filepath.length - 1);
-    if (found) {
-        arr.splice(i, 1);
-    }
-    return found;
-}
-
 function testFilePath(filepath) {
     try {
         if (filepath !== "-") {
